/*
Code by Pippadi.
Feel free to edit and distribute, although I doubt you would.
*/

#include <WiFi.h>
#include <PubSubClient.h>
#include <SSD1306Wire.h>
#include "TriangleWave.h"
#include "Alert.h"

WiFiClient wificlient;
PubSubClient client(wificlient);

const uint8_t oledClk = 5;   // OLED clock pin
const uint8_t oledData = 4;  // OLED data pin
const uint8_t oledAddr = 0x3c; // OLED I2C address
SSD1306Wire display(oledAddr, oledClk, oledData);

const char* SSID = "Your SSID"; // SSID (Super necessary comment, I know.)
const char* PASSWORD = "Wouldn't you like to know"; // WiFi password
const char* MQTTServer = "192.168.your.broker"; // MQTT broker address

uint8_t upBut = T6;          // "Up" button touch pin
uint8_t downBut = T4;        // "Down" button touch pin
uint8_t modeBut = T5;        // "Next" button touch pin
uint8_t redAlertBut = T3;    // "Alert" button touch pin
uint8_t touchThreshold = 30; // Greater the number, higher the sensitivity

uint8_t buzzer = 2;          // Buzzer Pin

uint8_t rPin = 16;           // Red Pin
uint8_t gPin = 25;           // Green Pin
uint8_t bPin = 26;           // Blue Pin

uint8_t buzzerChannel = 3;   // Buzzer PWM channel
uint8_t rChannel=0, gChannel=1, bChannel=2; // R, G, and B PWM channels
uint8_t channels[4] = {rChannel, gChannel, bChannel, buzzerChannel};

uint8_t lightVals[4] = {0, 0, 0, 0}; // R, G, B, Brightness
// R, G, and B values range from 0 to 25.
// Brightness ranges from 0 to 10.

volatile uint64_t lastActiveTime;
uint8_t currentMode = 0; // 0-Static Color, 1-Red Alert, 2-Random Color, 3-Blue Alert
bool unchanged;

bool upFlag = false;
bool downFlag = false;
bool modeFlag = false;
bool redAlertFlag = false;

void registerUp() {
  upFlag = true;
  lastActiveTime = millis();
}

void registerDown() {
  downFlag = true;
  lastActiveTime = millis();
}

void registerModeChange() {
  modeFlag = true;
  lastActiveTime = millis();
}

void registerRedAlert() {
  redAlertFlag = true;
  lastActiveTime = millis();
}

void setup() {
  ledcSetup(rChannel, 500, 8); // Red
  ledcSetup(gChannel, 500, 8); // Green
  ledcSetup(bChannel, 500, 8); // Blue
  ledcSetup(buzzerChannel, 2500, 8); // Buzzer
  ledcAttachPin(rPin, rChannel);
  ledcAttachPin(gPin, gChannel);
  ledcAttachPin(bPin, bChannel);
  ledcAttachPin(buzzer, buzzerChannel);
  display.init();
  display.flipScreenVertically();
  display.setFont(ArialMT_Plain_10);
  touchAttachInterrupt(redAlertBut, registerRedAlert, touchThreshold);
  touchAttachInterrupt(upBut, registerUp, touchThreshold);
  touchAttachInterrupt(downBut, registerDown, touchThreshold);
  touchAttachInterrupt(modeBut, registerModeChange, touchThreshold);
  lastActiveTime = millis();
}

void loop() {
  publishMode(currentMode);
  switch (currentMode) {
    case 0:
      drawDisplay(); 
      ledcWrite(rChannel, 255 - (lightVals[0]*lightVals[3]));
      ledcWrite(gChannel, 255 - (lightVals[1]*lightVals[3]));
      ledcWrite(bChannel, 255 - (lightVals[2]*lightVals[3]));
      waitForInput();
      break;
    case 1:
      handleRedAlert();
      break;
    case 2:
      handleRandomColor();
      break;
    case 3:
      handleBlueAlert();
      break;
  }
}